package HTML::Mason::Commands;
use strict;
use vars qw($m $r);
my $_escape = \&HTML::Mason::Parser::_escape_perl_expression;

my $widget = 'listManager';

my $get_my_meths = sub {
    my ($pkg, $field_titles, $field_values) = @_;
    my $meths = $pkg->my_meths();

    # Cook the display values
    while (my ($f, $t) = each %$field_titles) {
	$meths->{$f}->{'disp'} = $t;
    }

    # Cook the value methods
    if ($field_values) {
	foreach my $f (keys %$meths) {
	    my $meth = $meths->{$f}->{'get_meth'};

	    # Try to return a value from $field_values first.
	    my $cooked = sub { return ($field_values->($_[0], $f) ||
				       $meth->(@_)) };
	    
	    $meths->{$f}->{'get_meth'} = $cooked;
	}
    }

    return $meths;
};

sub build_table_data {
    my ($sort_objs, $meth, $fields, $select, $profile, $alter, $featured) = @_;
    my $data = [[map { $meth->{$_}->{'disp'} } @$fields]];
    my $cols = scalar @$fields;
    my $rows = 1 + scalar @$sort_objs;
    my $sel_cols = 0;
    my $prf_cols = 0;
    # Start at row 1 since we already have $fields loaded in $data
    my $r = 1;

    # Output the rows of data
    foreach my $o (@$sort_objs) {
	# Push the object id as the first value to be used in the listing comp.
	push @{$data->[$r]}, $o->get_id;

	# Load a flag to tell if this object is a featured object or not.
	my %flags = ('featured' => $featured->{$o->get_id});

	# Output for each field.
	foreach my $f (@$fields) {
	    my $val;
	    if ($meth->{$f}->{get_meth}) {
		# Try to call the get method.
		$val = $meth->{$f}->{get_meth}->($o,@{$meth->{$f}->{get_args}});
		# See if there is an existing alter method.
		$val = exists $alter->{$f} ? $alter->{$f}->($val, $o, \%flags)
                                           : $val;
	    }

	    # Add this value to the return data.
	    push @{$data->[$r]}, ($val || '&nbsp');
	}

	my @sel = output_select_controls($o, $select, \%flags);
	my @prf = output_profile_controls($o, $profile, \%flags);

	## Add the profile controls if any
	# MAX function
	$prf_cols = scalar @prf > $prf_cols ? scalar @prf : $prf_cols;

	push @{$data->[$r]}, @prf if @prf;
	
	## Add the select items if any
	if (@sel) {
	    $sel_cols = 1;
	    push @{$data->[$r]}, join('<br>', @sel);
	}

	$r++;
    }

    $cols += $sel_cols + $prf_cols;

    return ($rows, $cols, $data);
}

sub output_select_controls {
    my ($o, $select, $flags) = @_;
    my $vals = ref $select eq 'CODE' ? $select->($o, $flags) : $select;
    my @cntl;

    return unless $vals;

    # Turn this value into an array of arrays if it isn't already.
    $vals = ref($vals->[0]) eq 'ARRAY' ? $vals : [$vals];

    foreach my $v (@$vals) {
	my ($label, $name, $value) = @$v;
	$value ||= $o->get_id;

	push @cntl, $m->scomp('/widgets/profile/checkbox.mc', name  => $name,
	      	                                              value => $value).
		    $label;
    }

    return @cntl;
}

sub output_profile_controls {
    my ($o, $profile, $flags) = @_;
    my $vals = ref $profile eq 'CODE'  ? $profile->($o, $flags) : $profile;
    my @cntl;

    return unless $vals;

    # Turn this value into an array of arrays if it isn't already.
    $vals = ref($vals->[0]) eq 'ARRAY' ? $vals : [$vals];

    foreach my $v (@$vals) {
	my ($label, $url, $value) = @$v;

	# Don't set a default value if they passed the empty string.
	if ((not defined $value) or (length($value) > 0)) {
	    $value ||= 'id='.$o->get_id;
	    # Add the query string '?' if its not there already.
	    $value = "?$value" unless substr($value, 0, 1) eq '?';
	}

	push @cntl, "<a href='$url$value' class=redLink>$label</a>&nbsp;";
    }

    return @cntl;
}

sub build_constraints {
    my ($search_widget, $constrain, $meth, $sortBy) = @_;

    # Only get the criterion if we are still on the same page where it was set.
    my $prev = get_state_data($search_widget, 'crit_set_uri') || '';
    my $cur  = $r->uri || '';

    # Remove trailing slashes if they exsist.  Fixes a problem 
    # not realizing that '/foo/bar' and '/foo/bar/' are the same URL.
    substr($prev, -1, 1) = '' unless substr($prev, -1) ne '/';
    substr($cur, -1, 1)  = '' unless substr($cur, -1) ne '/';

    # Find the default search field.
    my $def_search_field;
    foreach my $f (keys %$meth) {
	# Break out of the loop if we find the searchable field.
	$def_search_field = $f and last if $meth->{$f}->{'search'};
    }

    # Initialize the sort column with the default search field.
    init_state_data($widget, 'sortBy', $sortBy || $def_search_field);

    my $crit = $cur eq $prev ? get_state_data($search_widget, 'criterion')
                             : undef;
    my $crit_field = get_state_data($search_widget, 'field');
    my $list_arg = {};

    # If any criteria were passed then we need to constrain our list.
    if ($crit && $crit_field) {
	# If field is an array, build a hash with the fields as the keys and 
        # $crit for vals
	if (ref $crit_field) {
	    @{$list_arg}{@$crit_field} = @$crit;
	} else {
	    $crit_field = $def_search_field if $crit_field eq '_default';
	    $list_arg->{$crit_field} = $crit;
	}
    }

    return $list_arg;
}

sub load_featured_objs {
    my ($objs, $pkg, $featured) = @_;
    # Find all loaded feature objects
    my %loaded = map { ($_->get_id, 1) } grep($featured->{$_->get_id}, @$objs);

    # Load any unloaded features.
    foreach (keys %$featured) {
	next if $loaded{$_};
	push @$objs, $pkg->lookup({'id' => $_});
    }

    return $objs;
}

sub sort_objects {
    my ($objs, $meth, $exclude) = @_;
    my @sort_objs;

    # Find which column to sort on.
    my $sort_by = get_state_data($widget, 'sortBy');

    # Only sort if the sort by was set in the state data.
    if ($sort_by) {
	# Make sure we pass an array ref to the sort arguments
	$sort_by = ref $sort_by ? $sort_by : [$sort_by];
	@sort_objs = sort { multisort($meth, @$sort_by) } @$objs;
    } else {
	@sort_objs = @$objs;
    }

    # Exclude objects with certain IDs.
    if ($exclude) {
	# Convert the exclude array into a HASH ref and return as a sub ref.
	if (ref $exclude eq 'ARRAY') {
	    my %h = map { $_ => '' } @$exclude;
	    $exclude = sub { exists $h{$_[0]->get_id} };
	}
	@sort_objs = grep(not($exclude->($_)), @sort_objs);
    }

    return @sort_objs;
}

sub multisort {
    my ($meth, @sort_list) = @_;
    my $sort_by               = shift @sort_list;
    my ($sort_get, $sort_arg) = @{$meth->{$sort_by}}{'get_meth', 'get_args'};

    # Do the case insensitive comparison
    my $val = lc($sort_get->($a, @$sort_arg)) cmp
              lc($sort_get->($b, @$sort_arg));

    # See if we need to do more comparisons or not.
    if (scalar(@sort_list) > 0) {
	return $val || multisort($meth, @sort_list);
    } else {
	return $val;
    }
}


HTML::Mason::Component::FileBased->new
(
'code'=>sub {
my %ARGS;
if (@_ % 2 == 0) { %ARGS = @_ } else { die "Odd number of parameters passed to component expecting name/value pairs" }
my $object = (!exists $ARGS{'object'} ?  die "no value sent for required parameter 'object'" : $ARGS{'object'});
my $style = (!exists $ARGS{'style'} ?  'full_list'  # The list style (full or paginated)
 : $ARGS{'style'});
my $title = (!exists $ARGS{'title'} ?  'Existing %n'# Text for the title of this list.
 : $ARGS{'title'});
my $sortBy = (!exists $ARGS{'sortBy'} ?  ''           # Default to sorting by ID
 : $ARGS{'sortBy'});
my $userSort = (!exists $ARGS{'userSort'} ?  1            # A flag for whether the user can resort the list
 : $ARGS{'userSort'});
my $profile = (!exists $ARGS{'profile'} ?  ['Edit', ''] # URL to the profile for this object.
 : $ARGS{'profile'});
my $select = (!exists $ARGS{'select'} ?  ['Delete', ''] # Add a checkbox
 : $ARGS{'select'});
my $addition = (!exists $ARGS{'addition'} ?  ['Add', '']  # Label and URL for adding an object to the list
 : $ARGS{'addition'});
my $search_widget = (!exists $ARGS{'search_widget'} ?  'search'     # Where to look for search criterion
 : $ARGS{'search_widget'});
my $fields = (!exists $ARGS{'fields'} ?  undef : $ARGS{'fields'});
my $field_titles = (!exists $ARGS{'field_titles'} ?  {} : $ARGS{'field_titles'});
my $field_values = (!exists $ARGS{'field_values'} ?  undef : $ARGS{'field_values'});
my $constrain = (!exists $ARGS{'constrain'} ?  {}           # Always constrain the search on a set of params
 : $ARGS{'constrain'});
my $behavior = (!exists $ARGS{'behavior'} ?  'narrow'     # How this list behaves.
 : $ARGS{'behavior'});
my $exclude = (!exists $ARGS{'exclude'} ?  undef           # Exclude certain objects from tjhe list.
 : $ARGS{'exclude'});
my $alter = (!exists $ARGS{'alter'} ?  {}           # Alter the data for one field
 : $ARGS{'alter'});
my $featured = (!exists $ARGS{'featured'} ?  undef        # Make one row a featured row
 : $ARGS{'featured'});
my $featured_color = (!exists $ARGS{'featured_color'} ?  '#cccc99'    # The color for the bkground of the featured row 
 : $ARGS{'featured_color'});
my $number = (!exists $ARGS{'number'} ?  0             : $ARGS{'number'});
my $objs = (!exists $ARGS{'objs'} ?  undef        # These are user objects to be listed.
 : $ARGS{'objs'});
my $_out = $m->current_sink;
$m->debug_hook($m->current_comp->path) if (%DB::);



#--------------------------------------#
# Initialize some values.

# Reset the state of this widget if the object name changes.
reset_state($widget, $object);

# Get the package name given the short name
my $pkg = get_package_name($object);

# Save the object type.
set_state_data($widget, 'object',   $object);
set_state_data($widget, 'pkg_name', $pkg);

# Get the master instance of this class.
my $meth = $get_my_meths->($pkg, $field_titles, $field_values);

# Set the fields to display
$fields ||= [sort keys %$meth];

# Set the title
my $name = get_class_info($object)->get_plural_name;
$title =~ s/\%n/$name/;

# We need a hash of featured IDs to use for later
my %featured_lookup = map { ($_,1) } @$featured;

#--------------------------------------#
# Find constraint and list objects.

my $list_arg = build_constraints($search_widget, $constrain, $meth, $sortBy);
my $param = {%$list_arg, %$constrain};

# Load the user provided objects into the @objs array.
my @objects;
@objects = @$objs if $objs;

my $empty_search;

# Only list if there are search parameters, or if our behaviour is 'narrow'.
if (!$objs && (scalar keys %$param or $behavior eq 'narrow')) {
    # Combine the list arguments and any passed constraints to search $pkg.
    @objects = $pkg->list($param);
} else {
    $empty_search = 1;
}

# Make sure our featured arguments are in the list
load_featured_objs(\@objects, $pkg, \%featured_lookup) if scalar(@$featured);

#--------------------------------------#
# Sort the objects.

my @sort_objs = sort_objects(\@objects, $meth, $exclude);

# Make sure we have some results.
my $no_results = (scalar(@sort_objs) == 0) && (scalar(keys(%$list_arg)) > 0);

#--------------------------------------#
# Build the table data array

my ($rows, $cols, $data) = build_table_data(\@sort_objs,
					    $meth,
					    $fields,
					    $select,
					    $profile,
					    $alter,
					    \%featured_lookup);

# Call the element to show this list
$m->comp("$style.mc", widget          => $widget,
	              title           => $title,
	              fields          => $fields,
	              data            => $data,
	              rows            => $rows,
	              pkg             => $pkg,
                      cols            => $cols,
                      userSort        => $userSort,
	 	      addition        => $addition,
	              featured        => \%featured_lookup,
	              featured_color  => $featured_color,
	              number          => $number,
	              empty_search    => $empty_search,
	);


$_out->('
');
$_out->('
');
$_out->('
');
$_out->('
');
return undef;

},
'create_time'=>1001666047,
'declared_args'=> {
          '$style' => {
                        'default' => ' \'full_list\'  # The list style (full or paginated)'
                      },
          '$search_widget' => {
                                'default' => ' \'search\'     # Where to look for search criterion'
                              },
          '$number' => {
                         'default' => ' 0            '
                       },
          '$object' => {
                         'default' => undef
                       },
          '$behavior' => {
                           'default' => ' \'narrow\'     # How this list behaves.'
                         },
          '$objs' => {
                       'default' => ' undef        # These are user objects to be listed.'
                     },
          '$title' => {
                        'default' => ' \'Existing %n\'# Text for the title of this list.'
                      },
          '$sortBy' => {
                         'default' => ' \'\'           # Default to sorting by ID'
                       },
          '$profile' => {
                          'default' => ' [\'Edit\', \'\'] # URL to the profile for this object.'
                        },
          '$field_values' => {
                               'default' => ' undef'
                             },
          '$fields' => {
                         'default' => ' undef'
                       },
          '$field_titles' => {
                               'default' => ' {}'
                             },
          '$featured' => {
                           'default' => ' undef        # Make one row a featured row'
                         },
          '$select' => {
                         'default' => ' [\'Delete\', \'\'] # Add a checkbox'
                       },
          '$userSort' => {
                           'default' => ' 1            # A flag for whether the user can resort the list'
                         },
          '$exclude' => {
                          'default' => ' undef           # Exclude certain objects from tjhe list.'
                        },
          '$alter' => {
                        'default' => ' {}           # Alter the data for one field'
                      },
          '$featured_color' => {
                                 'default' => ' \'#cccc99\'    # The color for the bkground of the featured row '
                               },
          '$constrain' => {
                            'default' => ' {}           # Always constrain the search on a set of params'
                          },
          '$addition' => {
                           'default' => ' [\'Add\', \'\']  # Label and URL for adding an object to the list'
                         }
        },
'object_size'=>14572,
'parser_version'=>0.8
)
;