package HTML::Mason::Commands;
use strict;
use vars qw($m $r);
my $_escape = \&HTML::Mason::Parser::_escape_perl_expression;

my $widget = 'select_object';

HTML::Mason::Component::FileBased->new
(
'code'=>sub {
my %ARGS;
if (@_ % 2 == 0) { %ARGS = @_ } else { die "Odd number of parameters passed to component expecting name/value pairs" }
my $style = (!exists $ARGS{'style'} ?  'dropdown' : $ARGS{'style'});
my $object = (!exists $ARGS{'object'} ?  die "no value sent for required parameter 'object'" : $ARGS{'object'});
my $name = (!exists $ARGS{'name'} ?  '' : $ARGS{'name'});
my $field = (!exists $ARGS{'field'} ?  die "no value sent for required parameter 'field'" : $ARGS{'field'});
my $constrain = (!exists $ARGS{'constrain'} ?  {} : $ARGS{'constrain'});
my $crit_field = (!exists $ARGS{'crit_field'} ?  undef : $ARGS{'crit_field'});
my $crit_value = (!exists $ARGS{'crit_value'} ?  undef : $ARGS{'crit_value'});
my $sort_field = (!exists $ARGS{'sort_field'} ?  undef : $ARGS{'sort_field'});
my $selected = (!exists $ARGS{'selected'} ?  undef : $ARGS{'selected'});
my $default = (!exists $ARGS{'default'} ?  undef : $ARGS{'default'});
my $no_persist = (!exists $ARGS{'no_persist'} ?  0 : $ARGS{'no_persist'});
my $width = (!exists $ARGS{'width'} ?  578 : $ARGS{'width'});
my $indent = (!exists $ARGS{'indent'} ?  FIELD_INDENT : $ARGS{'indent'});
my $disp = (!exists $ARGS{'disp'} ?  '' : $ARGS{'disp'});
my $useTable = (!exists $ARGS{'useTable'} ?  1 : $ARGS{'useTable'});
my $reset_key = (!exists $ARGS{'reset_key'} ?  undef : $ARGS{'reset_key'});
my $exclude = (!exists $ARGS{'exclude'} ?  [] : $ARGS{'exclude'});
my $readOnly = (!exists $ARGS{'readOnly'} ?  0 : $ARGS{'readOnly'});
my $req = (!exists $ARGS{'req'} ?  0 : $ARGS{'req'});
my $size = (!exists $ARGS{'size'} ?  undef : $ARGS{'size'});
my $js = (!exists $ARGS{'js'} ?  undef : $ARGS{'js'});
my $_out = $m->current_sink;
$m->debug_hook($m->current_comp->path) if (%DB::);



# Append the object name to the end of the widget name to allow multiple select
# boxes to exist on the same page.
my $sub_widget .= "$widget.$object";

# Convert the exclude array into a hash
$exclude = { map { $_ => '' } @$exclude };

# Reset this widget if the reset key changes.
reset_state($sub_widget, $reset_key);

if ($no_persist) {
    # Do not maintain selected state if the no_persist flag is set.
    set_state_data($sub_widget, 'selected_id', undef)
} else {
    # Set the default value if one has been passed.
    init_state_data($sub_widget, 'selected_id', $selected);
}

# Set the name we are supposed to use so that we can check it and repopulate.
set_state_data($sub_widget, 'form_name', $name);
# Grab the package name.
my $pkg = get_package_name($object);
my @vals;

# Handle the depreciated arguments 'crit_field' and 'crit_value'
if ($crit_field) {
    $constrain->{$crit_field} = $crit_value;
}

if ($pkg) {
    my $meth = $pkg->my_meths();
    my @objs = $pkg->list($constrain);

    # Sort the items if required.
    if ($sort_field) {
        my $get = $meth->{$sort_field}->{'get_meth'};
        my $arg = $meth->{$sort_field}->{'get_args'};

        @objs = sort { $get->($a, $arg) cmp $get->($b, $arg) } @objs;
    }

    # Put the default value at the top if it exists.
    $vals[0] = [@$default] if ($default);

    # Add the rest of the values from the object.
    my $val_get = $meth->{$field}->{'get_meth'};
    my $val_arg = $meth->{$field}->{'get_args'};

    foreach my $o (@objs) {
        my $id = $o->get_id;

	# Do not add excluded IDs.
	next if exists $exclude->{$id};

        my $val = $val_get->($o, $val_arg);
	
        push @vals, [$id, $val];
    } 
} else {
    # Handle the case where the package name is not in the database.
    @vals = ([0,"Pkg get failed: '$object'"]);
}

$m->comp($style.'.html', 
	 widget   => $widget, 
	 object   => $object, 
	 vals     => \@vals,
	 width    => $width,
	 indent   => $indent,
	 disp     => $disp,
	 useTable => $useTable,
	 readOnly => $readOnly,
	 size     => $size || @vals <= 20 ? 1 : 5,
	 req      => $req,
	 sel_id   => $selected,
	 js       => $js
);


#--- Documentation ---#
$_out->('
');
$_out->('
');
$_out->('
');
#--- Arguments ---#
$_out->('
');
$_out->('
');
#--- Initialization ---#
$_out->('
');
$_out->('
');
#--- Log History ---#
$_out->('
');
return undef;

},
'create_time'=>1001666047,
'declared_args'=> {
          '$crit_value' => {
                             'default' => ' undef'
                           },
          '$style' => {
                        'default' => ' \'dropdown\''
                      },
          '$size' => {
                       'default' => ' undef'
                     },
          '$readOnly' => {
                           'default' => ' 0'
                         },
          '$disp' => {
                       'default' => ' \'\''
                     },
          '$no_persist' => {
                             'default' => ' 0'
                           },
          '$object' => {
                         'default' => undef
                       },
          '$crit_field' => {
                             'default' => ' undef'
                           },
          '$req' => {
                      'default' => ' 0'
                    },
          '$selected' => {
                           'default' => ' undef'
                         },
          '$useTable' => {
                           'default' => ' 1'
                         },
          '$field' => {
                        'default' => undef
                      },
          '$sort_field' => {
                             'default' => ' undef'
                           },
          '$exclude' => {
                          'default' => ' []'
                        },
          '$reset_key' => {
                            'default' => ' undef'
                          },
          '$js' => {
                     'default' => ' undef'
                   },
          '$name' => {
                       'default' => ' \'\''
                     },
          '$indent' => {
                         'default' => ' FIELD_INDENT'
                       },
          '$constrain' => {
                            'default' => ' {}'
                          },
          '$width' => {
                        'default' => ' 578'
                      },
          '$default' => {
                          'default' => ' undef'
                        }
        },
'object_size'=>6372,
'parser_version'=>0.8
)
;